
/* Raspberry Pi light pen playground */
/* https://hackaday.io/project/172255-light-pen-support-for-retropie */
/* mwitkowiak@gmail.com */
/* feel free to use for any purpose */

#include <fcntl.h>
#include <poll.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

#include "fifo.h"

#include <wiringPi.h>

#define NGPIOS 1
#define LP_GPIO 17 // this is where light sensor is connected, pin 6 of DB9
#define BUT_GPIO 27 // 1 released, 0 pushed // button is connected here, pin 3 of DB9

int main(int argc, char **argv) {

if (wiringPiSetupGpio() == -1) {
    perror("wiring\n");
    return 1 ;
}
    pinMode(BUT_GPIO,INPUT);

    long totaldiff = 0;
    long totaln = 0;
    int lastbut = HIGH;
    long avgframe = 19900000/1000; // whole frame takes about this much

    long nframes = -1;

    struct timespec ts_real = (struct timespec) { 0,0 };
    struct timespec ts_real_last;
    struct timespec ts;
    long ts_zero;

    struct timespec ts_lplast = (struct timespec) { 0,0 };

    int files[NGPIOS];
    for (int i = 0; i < NGPIOS; ++i) {
        char gpio[64];
        snprintf(gpio, 64, "/dev/gpiots%d", i);
        int fd = open(gpio, O_RDONLY); 
        if (fd < 0) {
            printf("%s open error %d\n", gpio, fd);
            exit(-1);
        }
        files[i] = fd;
    }
    int n;
    struct pollfd fds[NGPIOS];
    for (int i = 0; i < NGPIOS; ++i) {
        fds[i].fd = files[i];
        fds[i].events = POLLPRI | POLLERR;
    }
    while (true) {
        int rc = poll(fds, NGPIOS, 2000);
        if (rc < 0) { // error
            perror("poll failed");
            return -1;
        }
        if (rc == 0) { // timeout
            //printf("poll timeout\n");
            continue;
        }
	// LP
	int i = 0;
	if (fds[0].revents!=0) { // LP event
	    n = read(files[i], &ts, 1);
//	    if (n==1) {
//		printf("error? n==1");
//	    } else {
                long usecs_start = ts_lplast.tv_sec * 1000000 + ts_lplast.tv_nsec / 1000;
                long usecs_end = ts.tv_sec * 1000000 + ts.tv_nsec / 1000;
                long micros = usecs_end - usecs_start;
		long diff = ts.tv_nsec - ts_lplast.tv_nsec;
		long diff_zero=0;
		ts_lplast=ts;
		if (diff>10000000) {
		    int but=digitalRead(BUT_GPIO);
//			clock_gettime(CLOCK_REALTIME, &ts_real);
//			long real_diff_frames = ((ts_real.tv_nsec-ts_real_last.tv_nsec)/1000)/avgframe;
//			ts_real_last = ts_real;
		    ts_zero = ts_zero+avgframe;
		    long real_diff_frames = (diff/1000)/avgframe;
//		    ts_zero = ts_zero + real_diff_frames*avgframe;
		    diff_zero = ts_zero - (ts.tv_nsec/1000);
		    long diff_frames = diff_zero/avgframe;
		    diff_zero = diff_zero-(diff_frames*avgframe);
//		    diff_zero = diff_zero-(real_diff_frames*avgframe);
//		    ts_zero = ts_zero + diff_frames*avgframe;
		    if (but==LOW) {
			if (lastbut==HIGH) {
			    lastbut=but;
			    totaldiff = 0;
			    totaln = 0;
			}
			totaldiff += diff;
			totaln++;
		    } else {
			if (lastbut==LOW) {
			    lastbut=but;
//			    avgframe = totaldiff/totaln/1000;
			    ts_zero = ts.tv_nsec/1000;
			    nframes=1;
			}
			if (nframes>0)
			    nframes++;
		    }
		    if (nframes>10) {
			nframes=1;
			printf("LP: %ld %i %i %i-%i diff=%ld\tavgframe=%ld\toffs=%ld\tyoffs=%ld\n",nframes,n,but,diff_frames,real_diff_frames,diff,avgframe,diff_zero,diff_zero/64);
		    }
		}
//	    }
	}
    }
    for (int i = 0; i < NGPIOS; ++i) {
        close(files[i]);
    }
    exit(0);
}
